--[[

This file controls the behavior of the ME units-list panel mods:
	Display mods:
		Added column to view bort numbers (first in group)
		Added column to view skins (liveries) for aircraft (first in group)
		Added column to view unit names (first in group)
	Button mods (additions):
		Hide All	- * hides all units  (* Only operates on the groups displayed in the panel window)
		Show All	- * shows all units (note: this operation is slow)
		Toggle		- * toggles hide/show property of all units
		Fix Borts	- resequences bort numbers of aircraft in one of two ways:
							linearly irrespective of aircraft type beginning with 'other' start number defined below or
							linearly within each aircraft type beginning with the start number defined below
							both methods are side-independent
		Fix Units	- renames units sequentially according to options set below.
		Run Script	- runs external Lua script file 'special.lua'
	Run Script:
		Resequences Eastern callsigns
		Print units list to ME log (set with options below)
		Print coalitions list to ME log (set with options below)
		Print group summary to ME log (set with options below)
		Print flyables list to ME log (set with options below)
		'Special.lua' is modifiable by the end-user and provides a hook into the ME for additional customization
		
Mods inspired by BlindSpot's ME mods for FC2/BS1

Special thanks to BlindSpot (ME mod), Speed (Mist and Slmod), and Panzertard (Servman) whose code
	showed me how things are done in Lua for DCS!

Enjoy! 159th_Ajax
	
--]]


me_mods_options = {


	-- self explanatory.
	-- turns functionality of new buttons on/off
	-- These are checked within me_units_list.lua
	
	["enableRunScript"] 			= true,
	["enableBortRenumbering"] 		= true,
	["enableUnitRenaming"] 			= true,

	
	-- determines scope of unit-renaming when 'Fix Units' button is clicked
	-- true = flyables will be renamed
	-- false = only non-flyables will be renamed
	
	["renameFlyables"] = false,

	
	
	-- determines method used to sequence borts when 'Fix Borts' button is clicked
	
	["fixBortsByType"] = false,

	
	
	-- starting bort numbers for re-sequencing 'ByType'
	
	["baseBortsByType"] = {
								["f15"] 	= 10,
								["su33"] 	= 20,
								["su27"] 	= 30,
								["mig29s"] 	= 40,
								["a10"] 	= 60,
								["ka50"] 	= 50,
								["mi8"] 	= 50,
								["uh1h"] 	= 50,
								["p51"] 	= 70,
								["su25"] 	= 80,
								["other"] 	= 10,		-- 'other' used as starting bort for non-'ByType' fix
								
	-- note to hackers: it's futile to try to add other types here unless you also re-code me_units_list.lua
	},

	
	
	-- WIP: re-assign callsigns
	---[[

	["reassignCallsigns"] 			= true,
	["reassignEasternCallsigns"] 	= true,
	["reassignWesternCallsigns"] 	= false,
	["baseEasternCallsigns"] 		= {
									["f15"] 	= 101,
									["su33"] 	= 151,
									["su27"] 	= 201,
									["mig29s"] 	= 251,
									["a10"] 	= 301,
									["ka50"] 	= 351,
									["p51"] 	= 401,
									["su25"] 	= 451,
									["mi8"] 	= 501,
									["uh1h"] 	= 551,
									["other"] 	= 601,
	},
	
	--]]



-- determines the suffix appended to unit names when 'Fix Units' button is clicked
	--  new name = groupname + sequential suffix
	--  Examples: assume group name = 'SochiDefenseGroup02'
	--	["suffixStyle"] = [[.%02i]],	-- Unit 1 is renamed to 'SochiDefenseGroup02.01'
	--	["suffixStyle"] = [[ #%03i]],	-- Unit 2 is renamed to 'SochiDefenseGroup02 #002'
	--	["suffixStyle"] = [[ %02i]],	-- Unit 3 is renamed to 'SochiDefenseGroup02 03'
	--	["suffixStyle"] = [[-%i]],		-- Unit 4 is renamed to 'SochiDefenseGroup02-4'
	--  see Lua reference 'string.format' for other possibilities

	["suffixStyle"] = [[-%02i]],		-- Unit 5 is renamed to 'SochiDefenseGroup02-05'
	
	
	
	-- ME log output options for Run Script function
	
	["runScriptOptions"] = {
								["printUnits"] 				= false,
								["printCoalitions"] 		= false,
								["printGroupSummary"] 		= false,
								["printFlyables"] 			= false,
	}
}
