local base = _G

module('me_units_list')

local require = base.require
local table = base.table
local pairs = base.pairs
local ipairs = base.ipairs
local tostring = base.tostring
local next = base.next

local DialogLoader = require('DialogLoader')
local Static = require('Static')
local GridHeaderCell = require('GridHeaderCell')
local U = require('me_utilities')
local MapWindow = require('me_map_window')
local MissionModule = require('me_mission')
local toolbar = require('me_toolbar')
local panel_aircraft = require('me_aircraft')
local panel_ship = require('me_ship')
local panel_vehicle = require('me_vehicle')
local panel_static = require('me_static')
local panel_route = require('me_route')
local CoalitionController	= require('Mission.CoalitionController')

local i18n = require('i18n')
i18n.setup(_M)

cdata = {
    title = _('GROUPS LIST'),
    helicopters = _('Helicopters'),
    planes = _('Planes'),
    vehicles = _('Vehicles'),
    ships = _('Ships'),
    statics = _('Statics'),
    object = _('NAME'),
    side = _('COUNTRY'),
    status = _('STATUS'),
    quantity = _('QNTY'),
    hidden = _('HIDDEN'),
    allCountries = _('ALL'),
    red = _('Red'),
    blue = _('Blue'),  
    Total = _('Total:'),    
    units = _('units'),   
}

local unitsNum = {}
unitsNum['helicopter']= 0
unitsNum['plane']= 0
unitsNum['vehicle']= 0
unitsNum['ship']= 0
unitsNum['static']= 0

function applyFilter(group)
    if not group then
        return false
    end
    
    local coalitionName = group.boss.boss.name
    local countryName = group.boss.name
    
    if filters[group.type]:getState() then
        local comboText = countryFilter:getText()
        
        if comboText == cdata.allCountries then
            return true
        end
        if (comboText == cdata.red) and (coalitionName == CoalitionController.redCoalitionName()) then
            return true
        end
        if (comboText == cdata.blue) and (coalitionName == CoalitionController.blueCoalitionName()) then
            return true
        end
        if (comboText == countryName) then
            return true
        end
        return false
    else
        return false
    end
end

function createColumnInfo(name, width)
  return {name = name, width = width}
end

function initColumnInfos()
    --     
    columnInfos = {
        createColumnInfo(cdata.object, 400),
        createColumnInfo(cdata.side, 100),
        createColumnInfo(cdata.status, 100),
        createColumnInfo(cdata.quantity, 50),
    }
end

function getColumnInfoCount()
  return #columnInfos
end

function getColumnInfo(index)
  return columnInfos[index]
end

function createGridHeader()
    initColumnInfos()
    
    local columnInfoCount = getColumnInfoCount()
    
    for i = 1, columnInfoCount do
        local columnInfo = getColumnInfo(i)
        local headerCell = GridHeaderCell.new(columnInfo.name)

        grid:insertColumn(columnInfo.width, headerCell)
    end 
end

function createGrid()
  createGridHeader()
  
    --    
    function grid:onMouseDown(x, y, button)
        if 1 == button then
            local col, row = self:getMouseCursorColumnRow(x, y)

            if -1 < col and -1 < row then
        self:selectRow(row)
        
        local cell = self:getCell(col, row)
        
        if cell then
            onGridMouseDown(cell)
        end
            end
        end
    end    
    
    function grid:onMouseDoubleClick(x, y, button)
        if 1 == button then
            local col, row = self:getMouseCursorColumnRow(x, y)
      
            if -1 < col and -1 < row then
        self:selectRow(row)
        
        local cell = self:getCell(col, row)
        
        if cell then
            onMouseDoubleClick(cell)
        end
            end
        end        
    end   
end

local function addFilter(filters, filterType, checkBox)
    checkBox.onChange = update
    filters[filterType] = checkBox
end

local function createFilters()
    filters = {}
    
    addFilter(filters, 'helicopter', window.checkBoxHelicopters)
    addFilter(filters, 'plane',  window.checkBoxPlanes)
    addFilter(filters, 'vehicle', window.checkBoxVehicles)
    addFilter(filters, 'ship', window.checkBoxShips)
    addFilter(filters, 'static', window.checkBoxStatics)
end

function create(x, y, w, h)
    window = DialogLoader.spawnDialogFromFile(base.dialogsDir .. "me_units_list_panel.dlg", cdata)
    window:setBounds(x, y, w, h)
    
    grid = window.grid
    
    local cw, ch = window:getClientRectSize()
    local gridX, gridY = grid:getPosition()
    
    grid:setSize(cw - gridX, ch - gridY)
    
    cellSkin = window.staticCell:getSkin()
    
    countryFilter = window.countryFilter
    
    createFilters()
    
    updateCountriesCombo()
    countryFilter:setText(cdata.allCountries)
    countryFilter.onChange = update

    function window:onClose()
        show(false)
        toolbar.setUnitListButtonState(false)
    end
    
    createGrid()
    
    
    window:addHotKeyCallback("Up", onGridUp)
    window:addHotKeyCallback("Down", onGridDown)
end

function onGridUp()
    local index = grid:getSelectedRow()
    if index > 0 then
        grid:selectRow(index-1)
        
        local cell = grid:getCell(0, index-1)
        
        if cell then
            onGridMouseDown(cell)
        end    
    end    
end

function onGridDown()
    local index = grid:getSelectedRow()
    if index < (grid:getRowCount()-1) then
        grid:selectRow(index-1)
        
        local cell = grid:getCell(0, index+1)
        
        if cell then
            onGridMouseDown(cell)
        end    
    end 
end


function show(b)
    if b then
        updateCountriesCombo()
        update()
        
        if MapWindow.selectedGroup then
            selectGroup(MapWindow.selectedGroup)
        else 
            --   
            --      
            --    
            local k = next(MissionModule.group_by_name)
            
            if k then 
                local group = MissionModule.group_by_name[k]
                
                if group and (not group.hidden) then 
                    selectGroup(group)
                    selectRow(group)
                end
            else
                window:setVisible(false)
                toolbar.setUnitListButtonState(false)
                
                return
            end
        end
    else
        toolbar.setUnitListButtonState(false)
    end
    
    window:setVisible(b)
end

function updateStatistic()
    window.s_helicopters:setText(unitsNum['helicopter'])
    window.s_planes:setText(unitsNum['plane'])
    window.s_vehicles:setText(unitsNum['vehicle'])
    window.s_ships:setText(unitsNum['ship'])
    window.s_statics:setText(unitsNum['static'])
    window.s_total:setText(unitsNum['helicopter']+unitsNum['plane']+unitsNum['vehicle']+unitsNum['ship']+unitsNum['static'])
end

local function createGroupByNameTable()
    local group_by_name = {}
    unitsNum['helicopter']= 0
    unitsNum['plane']= 0
    unitsNum['vehicle']= 0
    unitsNum['ship']= 0
    unitsNum['static']= 0
    
    for r,group in pairs(MissionModule.group_by_name) do
        if group then 
            table.insert(group_by_name,group)             
            unitsNum[group.type] = unitsNum[group.type] + #group.units
        end
    end
    
    updateStatistic()    
    local sortGroupByName = function(group1, group2)
        return group1.name < group2.name
    end
    
    table.sort(group_by_name, sortGroupByName)
    
    return group_by_name
end

local function getHiddenString(group)
  local result = ''
            
  if group.hidden then 
      result = cdata.hidden
  end
  
  return result
end

local function fillGridRow(group)
    -- FIXME:     
    local rowHeight = 20
    
    grid:insertRow(rowHeight)
    
    local rowIndex = grid:getRowCount() - 1
  
    local text = {group.name, group.boss.name, getHiddenString(group), #group.units}            
    local columnInfoCount = getColumnInfoCount()
    
    for columnIndex = 0, columnInfoCount - 1 do
        local cell = Static.new(text[columnIndex + 1])
        
        cell:setSkin(cellSkin)
        cell.data = {group = group, r = rowIndex, c = columnIndex}

        grid:setCell(columnIndex, rowIndex, cell)
    end
end

function update()    
    --  
    grid:clearRows()

    local group_by_name = createGroupByNameTable()
    
    for r, group in pairs(group_by_name) do
        if group and applyFilter(group) then 
            fillGridRow(group)
        end
    end
    
    -- 
    if MapWindow.selectedGroup and MapWindow.selectedGroup.boss then 
        selectRow(MapWindow.selectedGroup)
    end
end

-------------------------------------------------------------------------------
--
function onMouseDoubleClick(self)
    self.data.group.hidden = not self.data.group.hidden
    
    --   
    updateRow(self.data.group)
    if self.data.group.hidden then --  ,             
        MissionModule.remove_group_map_objects(self.data.group)
    else   --  ,     
        MissionModule.create_group_map_objects(self.data.group)
        --    
        selectGroup(self.data.group)
    end
    onGridMouseDown(self)
end

--     
function onGridMouseDown(self, x, y, button)
    if not applyFilter(self.data.group) then 
        return
    end

    selectGroup(self.data.group)
    selectRow(self.data.group, self.data.r)
end

--    
function selectGroup(group)
    if (not group) or (not group.boss) or (not group.boss.boss) or (not applyFilter(group)) then 
        return
    end
    
    if group.hidden then
        MapWindow.unselectAll()
        MapWindow.selectedGroup = group
        setCheckBox(group.type, true)
        local selectedUnit = group.units[1]
        local mapObject = group.mapObjects.units[1]
        MapWindow.selectedUnit = selectedUnit
        MapWindow.respondToSelectedUnit(mapObject, group, selectedUnit)
        return
    end
    
    local mapObject
    
    if group.mapObjects.route then
        mapObject = group.mapObjects.route.points[1]
    else
        mapObject = group.mapObjects.units[1].userObject
        mapObject.userObject = group.mapObjects.units[1].userObject
    end
    
    setCheckBox(group.type, false)
    
    if not mapObject then 
        return
    end
    
    local unit = group.mapObjects.units[1].userObject
    
    MapWindow.unselectAll()
    
    local selectedUnit = group.units[1]
    
    MapWindow.respondToSelectedUnit(mapObject, group, selectedUnit)
    MapWindow.updateSelectedGroup(group)
    panel_route.update()
    
    if window:isVisible() then
        --    
        --           ,
        --        
        MapWindow.setCamera(group.x, group.y)
        
        local mx,my,mw,mh = MapWindow.window:getBounds()
        local _x,_y = MapWindow.window:widgetToWindow((mw - U.right_toolbar_width)/2,(mh - 300)/2)
        local x, y =  MapWindow.getMapPoint(_x,_y)
        
        MapWindow.setCamera(2*group.x - x, 2*group.y - y)        
    end
    
end

function getGroupRowIndex(group)
  local result
  
  for rowIndex = 0, grid:getRowCount() - 1 do 
      local widget = grid:getCell(0, rowIndex)
      
      if widget.data.group == group then 
          result = rowIndex
          
          break
      end
  end
  
  return result
end

--    
function selectRow(group, rowIndex)
    if not applyFilter(group) then 
        grid:selectRow(-1)
        
        return
    end
    
    if not rowIndex then 
        rowIndex = getGroupRowIndex(group)
        
        if rowIndex then
            grid:selectRow(rowIndex)
            selection = rowIndex          
        end
    else
        grid:selectRow(rowIndex)
    end
end

--    
function updateRow(group)
    if not applyFilter(group) then 
        return
    end
    
    local rowIndex = getGroupRowIndex(group)
    
    if rowIndex then
        local widget = grid:getCell(0, rowIndex)
        
        if widget.data.group == group then 
            widget:setText(group.name)
            
            widget = grid:getCell(1, rowIndex)
            widget:setText(group.boss.name)
            
            widget = grid:getCell(2, rowIndex)
            
            local hiddenStr = ''
            
            if group.hidden then 
                hiddenStr = cdata.hidden 
            end
            
            widget:setText(hiddenStr)
            
            widget = grid:getCell(3, rowIndex)
            widget:setText(tostring(#group.units))
        end
    end
end

--      
function setCheckBox(type, b)
    if type == 'plane' or
       type == 'helicopter' then
        panel_aircraft.hiddenCheckbox:setState(b)
    elseif type == 'ship' then
        panel_ship.hiddenCheckbox:setState(b)
    elseif type == 'vehicle' then
        panel_vehicle.hiddenCheckbox:setState(b)
    elseif type == 'static' then
        panel_static.hiddenCheckbox:setState(b)
    end
end

function selectNextGroup()
    local rowCount = grid:getRowCount()
    
    if -1 ~= selection then
        if selection > rowCount - 1 then
            selection = rowCount - 1
        end
        
        if -1 ~= selection and (selection >= 0)then
            local w = grid:getCell(0, selection)
            local data = w.data
            
            selectGroup(data.group)
            selectRow(data.group, selection)
        else
            show(false)
            MapWindow.selectedGroup = nil
            toolbar.untoggle_all_except()        
        end
    end   
end

function saveSelection()
    selection = grid:getSelectedRow()
end

function updateCountriesCombo()
    if MissionModule.mission == nil then
        return
    end
    
    countryFilter:clear()
	
	local countries = CoalitionController.getActiveCountries()
	
    table.insert(countries, 1, cdata.allCountries)
    table.insert(countries, 2, cdata.red)
    table.insert(countries, 3, cdata.blue)	
	
    U.fill_combo(countryFilter, countries)
end 
