local base = _G

module('me_units_list')

local require = base.require
local table = base.table
local pairs = base.pairs
local ipairs = base.ipairs
local tostring = base.tostring
local next = base.next
-- added by 159th_Ajax
local string = base.string
local print = base.print
local type = base.type
local loadfile = base.loadfile

local DialogLoader = require('DialogLoader')
local Static = require('Static')
local GridHeaderCell = require('GridHeaderCell')
local U = require('me_utilities')
local MapWindow = require('me_map_window')
local MissionModule = require('me_mission')
local toolbar = require('me_toolbar')
local panel_aircraft = require('me_aircraft')
local panel_ship = require('me_ship')
local panel_vehicle = require('me_vehicle')
local panel_static = require('me_static')
local panel_route = require('me_route')
local CoalitionController	= require('Mission.CoalitionController')

local i18n = require('i18n')
i18n.setup(_M)

cdata = {
    title = _('GROUPS LIST'),
    helicopters = _('Helicopters'),
    planes = _('Planes'),
    vehicles = _('Vehicles'),
    ships = _('Ships'),
    statics = _('Statics'),
    object = _('NAME'),
    side = _('COUNTRY'),
    status = _('STATUS'),
    quantity = _('QNTY'),
    hidden = _('HIDDEN'),
    allCountries = _('ALL'),
    red = _('Red'),
    blue = _('Blue'),  
	
-- 159th_Ajax mod
---[[
	col_skin = _('SKIN'),
	col_name = _('UNIT NAME'),
	col_bort = _('BORT'),
    showAll = _('Show all'),
    hideAll = _('Hide all'),
    toggle = _('Toggle'),
    fixBorts = _('Fix Borts'),
    fixUnits = _('Fix Units'),
    special = _('Run Script'),
--]] end mod

    Total = _('Total:'),    
    units = _('units'),   
}

local unitsNum = {}
unitsNum['helicopter']= 0
unitsNum['plane']= 0
unitsNum['vehicle']= 0
unitsNum['ship']= 0
unitsNum['static']= 0

function applyFilter(group)
    if not group then
        return false
    end
    
    local coalitionName = group.boss.boss.name
    local countryName = group.boss.name
    
    if filters[group.type]:getState() then
        local comboText = countryFilter:getText()
        
        if comboText == cdata.allCountries then
            return true
        end
        if (comboText == cdata.red) and (coalitionName == CoalitionController.redCoalitionName()) then
            return true
        end
        if (comboText == cdata.blue) and (coalitionName == CoalitionController.blueCoalitionName()) then
            return true
        end
        if (comboText == countryName) then
            return true
        end
        return false
    else
        return false
    end
end

function createColumnInfo(name, width)
  return {name = name, width = width}
end

function initColumnInfos()
    --     
    columnInfos = {
        createColumnInfo(cdata.object, 400),
        createColumnInfo(cdata.side, 100),
        createColumnInfo(cdata.status, 100),
        createColumnInfo(cdata.quantity, 50),
		
		-- 159th_Ajax mod
		---[[
        createColumnInfo(cdata.col_bort, 50),
        createColumnInfo(cdata.col_skin, 300),
        createColumnInfo(cdata.col_name, 300),
		--]] end mod
    }
end

function getColumnInfoCount()
  return #columnInfos
end

function getColumnInfo(index)
  return columnInfos[index]
end

function createGridHeader()
    initColumnInfos()
    
    local columnInfoCount = getColumnInfoCount()
    
    for i = 1, columnInfoCount do
        local columnInfo = getColumnInfo(i)
        local headerCell = GridHeaderCell.new(columnInfo.name)

        grid:insertColumn(columnInfo.width, headerCell)
    end 
end

function createGrid()
  createGridHeader()
  
    --    
    function grid:onMouseDown(x, y, button)
        if 1 == button then
            local col, row = self:getMouseCursorColumnRow(x, y)

            if -1 < col and -1 < row then
        self:selectRow(row)
        
        local cell = self:getCell(col, row)
        
        if cell then
            onGridMouseDown(cell)
        end
            end
        end
    end    
    
    function grid:onMouseDoubleClick(x, y, button)
        if 1 == button then
            local col, row = self:getMouseCursorColumnRow(x, y)
      
            if -1 < col and -1 < row then
        self:selectRow(row)
        
        local cell = self:getCell(col, row)
        
        if cell then
            onMouseDoubleClick(cell)
        end
            end
        end        
    end   
end

local function addFilter(filters, filterType, checkBox)
    checkBox.onChange = update
    filters[filterType] = checkBox
end

local function createFilters()
    filters = {}
    
    addFilter(filters, 'helicopter', window.checkBoxHelicopters)
    addFilter(filters, 'plane',  window.checkBoxPlanes)
    addFilter(filters, 'vehicle', window.checkBoxVehicles)
    addFilter(filters, 'ship', window.checkBoxShips)
    addFilter(filters, 'static', window.checkBoxStatics)
end

function create(x, y, w, h)
    window = DialogLoader.spawnDialogFromFile(base.dialogsDir .. "me_units_list_panel.dlg", cdata)
    window:setBounds(x, y, w, h)
    
    grid = window.grid
    
    local cw, ch = window:getClientRectSize()
    local gridX, gridY = grid:getPosition()
    
    grid:setSize(cw - gridX, ch - gridY)
    
    cellSkin = window.staticCell:getSkin()
    
    countryFilter = window.countryFilter
    
    createFilters()
    
    updateCountriesCombo()
    countryFilter:setText(cdata.allCountries)
    countryFilter.onChange = update

    function window:onClose()
        show(false)
        toolbar.setUnitListButtonState(false)
    end
    
	-- 159th_Ajax mod
	---[[
    setupCallbacks() 
	--]] end mod
	
    createGrid()

	
    window:addHotKeyCallback("Up", onGridUp)
    window:addHotKeyCallback("Down", onGridDown)
end

function onGridUp()
    local index = grid:getSelectedRow()
    if index > 0 then
        grid:selectRow(index-1)
        
        local cell = grid:getCell(0, index-1)
        
        if cell then
            onGridMouseDown(cell)
        end    
    end    
end

function onGridDown()
    local index = grid:getSelectedRow()
    if index < (grid:getRowCount()-1) then
        grid:selectRow(index-1)
        
        local cell = grid:getCell(0, index+1)
        
        if cell then
            onGridMouseDown(cell)
        end    
    end 
end

--159th_Ajax mod
---[[
function setupCallbacks()
    function window.buttonShow:onChange()
		changeAll(true)
	end
	function window.buttonHide:onChange()
		changeAll(false)
	end
	function window.buttonToggle:onChange()
		changeAll()
	end
	function window.buttonFixBorts:onChange()
		fixBorts()
	end
	function window.buttonFixUnits:onChange()
		fixUnits()
	end
	function window.buttonSpecial:onChange()
		specialScript()
	end
end

--copy of createGroupByNameTable() function below
function getGroupByNameTable()
    local group_by_name = {}
   
    for r,group in pairs(MissionModule.group_by_name) do
        if group then 
            table.insert(group_by_name,group) 
        end
    end
    
    local sortGroupByName = function(group1, group2)
		-- sorts by coalition (desc), groupname (asc)
		if group1.boss.boss.name == group2.boss.boss.name then
			return group1.name < group2.name
		else
			return group1.boss.boss.name > group2.boss.boss.name
		end
--        return group1.name < group2.name
    end
    
    table.sort(group_by_name, sortGroupByName)
    
    return group_by_name
end

function assignBortsByType(_n, unit)

	if unit.type == 'F-15C' then
		unit.onboard_num = string.format("%03d",tostring(_n.f15))
		_n.f15 = _n.f15 + 1
	elseif unit.type == 'P-51D' then
		unit.onboard_num = string.format("%03d",tostring(_n.p51))
		_n.p51 = _n.p51 + 1
	elseif unit.type == 'Ka-50' then
		unit.onboard_num = string.format("%03d",tostring(_n.ka50))
		_n.ka50 = _n.ka50 + 1
	elseif unit.type == 'Mi-8MT' then
		unit.onboard_num = string.format("%03d",tostring(_n.mi8))
		_n.mi8 = _n.mi8 + 1
	elseif unit.type == 'UH-1H' then
		unit.onboard_num = string.format("%03d",tostring(_n.uh1h))
		_n.uh1h = _n.uh1h + 1
	elseif unit.type == 'Su-33' then
		unit.onboard_num = string.format("%03d",tostring(_n.su33))
		_n.su33 = _n.su33 + 1
	elseif unit.type == 'Su-27' then
		unit.onboard_num = string.format("%03d",tostring(_n.su27))
		_n.su27 = _n.su27 + 1
	elseif unit.type == 'MiG-29S' then
		unit.onboard_num = string.format("%03d",tostring(_n.mig29s))
		_n.mig29s = _n.mig29s + 1
	elseif unit.type == 'A-10C' then
		unit.onboard_num = string.format("%03d",tostring(_n.a10))
		_n.a10 = _n.a10 + 1
	elseif unit.type == 'Su-25' or unit.type == 'Su-25T' then
		unit.onboard_num = string.format("%03d",tostring(_n.su25))
		_n.su25 = _n.su25 + 1
	else -- all others
		unit.onboard_num = string.format("%03d",tostring(_n.other))
		_n.other = _n.other + 1
	end

end

function assignBorts(_n,unit)
	unit.onboard_num = string.format("%03d",tostring(_n.other))
	_n.other = _n.other + 1
end

function fixBorts()

	local modsOptions = {}
	local f = base.loadfile(base.mainPath.."modules/me_mods_options.lua")
	local env = {}
	if f then
		base.setfenv(f,env)
		f()
		modsOptions = env.me_mods_options
	end
	if not modsOptions.enableBortRenumbering then return end
	local type = base.type
	local baseBortsDefaults = {}
	if modsOptions.baseBortsByType then
		baseBortsDefaults = modsOptions.baseBortsByType 
	else
		baseBortsDefaults = {
			["f15"] 	= 10,
			["su33"] 	= 20,
			["su27"] 	= 30,
			["mig29s"] 	= 40,
			["a10"] 	= 60,
			["ka50"] 	= 50,
			["mi8"] 	= 50,
			["uh1h"] 	= 50,
			["p51"] 	= 80,
			["su25"] 	= 70,
			["other"] 	= 10,
		}
	end
	local base_bort = {}
	base_bort["blue"] = {}
	base_bort["red"] = {}
	base_bort["neutr"] = {}
	for i,v in pairs(baseBortsDefaults) do
	-- copy to each side for independent numbering
		base_bort.red[i] = v
		base_bort.blue[i] = v
		base_bort.neutr[i] = v
	end
	local _tmp = getGroupByNameTable()
	for group_idx, group_data in pairs(_tmp) do
		if type(group_data) == 'table' then
			local _n = {}
			if group_data.boss.boss.name == 'BLUE' then
				_n = base_bort.blue
			elseif group_data.boss.boss.name == 'RED' then
				_n = base_bort.red
			else
				_n = base_bort.neutr
			end
			if (group_data.type == 'helicopter' or group_data.type == 'plane') then
				for i,v in pairs(group_data.units) do
					if modsOptions.fixBortsByType then
						assignBortsByType(_n,v) --assigns sequentially within each aircraft type
					else
						assignBorts(_n,v) --alternate method assigns Borts sequentially irrespective of type
					end
				end
			end
		end
	end
end

function unitCompare(unit1,unit2)
	if unit1.coalition == unit2.coalition then
		if unit1.group == unit2.group then
			return unit1.name < unit2.name
		else
			return unit1.group < unit2.group
		end
	else
		return unit1.coalition > unit2.coalition	-- reverse order
	end
end

function getUnitsTable()
	local _tmp = {}
	for unit_id, unit_data in pairs(MissionModule.unit_by_id) do
		if base.type(unit_data) == 'table' then
			local _n = #_tmp + 1
			_tmp[_n] = {}
			_tmp[_n].index 			= unit_data.index
			_tmp[_n].livery_id	 	= unit_data.livery_id or ""
			_tmp[_n].skill 			= unit_data.skill or ""
			_tmp[_n].group 			= unit_data.boss.name
			_tmp[_n].group_type 	= unit_data.boss.type
			_tmp[_n].group_hidden 	= unit_data.boss.hidden or false
			_tmp[_n].country 		= unit_data.boss.boss.name
			_tmp[_n].coalition 		= unit_data.boss.boss.boss.name
			_tmp[_n].type 			= unit_data.type
			_tmp[_n].unitId			= unit_data.unitId
			_tmp[_n].name 			= unit_data.name
			_tmp[_n].onboard_num 	= unit_data.onboard_num or ""
			_tmp[_n].callsign 		= unit_data.callsign or ""
			_tmp[_n].group_task		= unit_data.boss.task or ""
		end
	end
	return _tmp
end

function fixUnits()
	local modsOptions = {}
	local f = base.loadfile(base.mainPath.."modules/me_mods_options.lua")
	local env = {}
	if f then
		base.setfenv(f,env)
		f()
		modsOptions = env.me_mods_options
	end
	if not modsOptions.enableUnitRenaming then return end
	local renameFlyables = false
	if modsOptions.renameFlyables ~= nil then
		renameFlyables = modsOptions.renameFlyables
	end
	local suffixStyle = ".%03i"
	if modsOptions.suffixStyle ~= nil then
		suffixStyle = modsOptions.suffixStyle
	end
	local _tmp = getUnitsTable()
	table.sort(_tmp,unitCompare)
	for i,v in pairs(_tmp) do
		local oldname = v.name
		local newname = v.group..string.format(suffixStyle,v.index)
		if renameFlyables then
			if not MissionModule.unit_by_name[newname] then
				MissionModule.renameUnit(MissionModule.unit_by_id[v.unitId],newname)
			end
		elseif v.skill ~= 'Client' and v.skill ~= 'Player' then
			if not MissionModule.unit_by_name[newname] then
				MissionModule.renameUnit(MissionModule.unit_by_id[v.unitId],newname)
			end
		end
	end
end

function specialScript()

---[[
	--dynamically loads and compiles script in 'special.lua' in \missioneditor\modules\
	--useful for inspecting data in ME context

	--check to see if option is enabled
	local modsOptions = {}
	local f = loadfile(base.mainPath.."modules/me_mods_options.lua")
	local env = {}
	if f then
		base.setfenv(f,env)
		f()
		modsOptions = env.me_mods_options
	end
	if not modsOptions.enableRunScript then return end
	
	--then load script
	f = loadfile(base.mainPath.."modules/special.lua")
	env = {base = base}
	if f then
		base.setfenv(f,env)
		f()
	end
	
--]]

end

-- From Blindspot's ME mod for FC2/BS1
function changeAll(b)

	local rowCount=grid:getRowCount()
    for i=1,rowCount do
        local w = grid:getCell(1, i-1);
        local group = w.data.group;
        if true == b then
            if group.hidden then
                MissionModule.create_group_map_objects(group)
				selectGroup(group);
            end;
            group.hidden = false;
        elseif false == b then
            if not group.hidden then
                MissionModule.remove_group_map_objects(group)
            end;
            group.hidden = true;
        else
            if group.hidden then
                MissionModule.create_group_map_objects(group)
				selectGroup(group);
            else
                MissionModule.remove_group_map_objects(group)
            end;
            group.hidden = not group.hidden;
        end;
		updateRow(group);
    end;
end;
--]]
-- end mod

function show(b)
    if b then
        updateCountriesCombo()
        update()
        
        if MapWindow.selectedGroup then
            selectGroup(MapWindow.selectedGroup)
        else 
            --   
            --      
            --    
            local k = next(MissionModule.group_by_name)
            
            if k then 
                local group = MissionModule.group_by_name[k]
                
                if group and (not group.hidden) then 
                    selectGroup(group)
                    selectRow(group)
                end
            else
                window:setVisible(false)
                toolbar.setUnitListButtonState(false)
                
                return
            end
        end
    else
        toolbar.setUnitListButtonState(false)
    end
    
    window:setVisible(b)
end

function updateStatistic()
    window.s_helicopters:setText(unitsNum['helicopter'])
    window.s_planes:setText(unitsNum['plane'])
    window.s_vehicles:setText(unitsNum['vehicle'])
    window.s_ships:setText(unitsNum['ship'])
    window.s_statics:setText(unitsNum['static'])
    window.s_total:setText(unitsNum['helicopter']+unitsNum['plane']+unitsNum['vehicle']+unitsNum['ship']+unitsNum['static'])
end

local function createGroupByNameTable()
    local group_by_name = {}
    unitsNum['helicopter']= 0
    unitsNum['plane']= 0
    unitsNum['vehicle']= 0
    unitsNum['ship']= 0
    unitsNum['static']= 0
    
    for r,group in pairs(MissionModule.group_by_name) do
        if group then 
            table.insert(group_by_name,group)             
            unitsNum[group.type] = unitsNum[group.type] + #group.units
        end
    end
    
    updateStatistic()    
    local sortGroupByName = function(group1, group2)
        return group1.name < group2.name
    end
    
    table.sort(group_by_name, sortGroupByName)
    
    return group_by_name
end

local function getHiddenString(group)
  local result = ''
            
  if group.hidden then 
      result = cdata.hidden
  end
  
  return result
end

local function fillGridRow(group)
    -- FIXME:     
    local rowHeight = 20
    
    grid:insertRow(rowHeight)
    
    local rowIndex = grid:getRowCount() - 1
  
	--original
    --local text = {group.name, group.boss.name, getHiddenString(group), #group.units}            
	
	-- 159th_Ajax mod
	---[[
	local unit_skin = ""
	local unit_name = ""
	local unit_bort = ""
	if nil ~= group.units[1] then
		if nil ~= group.units[1].livery_id then
			unit_skin = group.units[1].livery_id
		end
		if nil ~= group.units[1].name then
			unit_name = group.units[1].name
		end
		if nil ~= group.units[1].onboard_num then
			unit_bort = group.units[1].onboard_num
		end
	end
    local text = {group.name, group.boss.name, getHiddenString(group), #group.units, unit_bort, unit_skin, unit_name}
	--]] end mod
	
    local columnInfoCount = getColumnInfoCount()
    
    for columnIndex = 0, columnInfoCount - 1 do
        local cell = Static.new(text[columnIndex + 1])
        
        cell:setSkin(cellSkin)
        cell.data = {group = group, r = rowIndex, c = columnIndex}

        grid:setCell(columnIndex, rowIndex, cell)
    end
end

function update()    
    --  
    grid:clearRows()

    local group_by_name = createGroupByNameTable()
    
    for r, group in pairs(group_by_name) do
        if group and applyFilter(group) then 
            fillGridRow(group)
        end
    end
    
    -- 
    if MapWindow.selectedGroup and MapWindow.selectedGroup.boss then 
        selectRow(MapWindow.selectedGroup)
    end
end

-------------------------------------------------------------------------------
--
function onMouseDoubleClick(self)
    self.data.group.hidden = not self.data.group.hidden
    
    --   
    updateRow(self.data.group)
    if self.data.group.hidden then --  ,             
        MissionModule.remove_group_map_objects(self.data.group)
    else   --  ,     
        MissionModule.create_group_map_objects(self.data.group)
        --    
        selectGroup(self.data.group)
    end
    onGridMouseDown(self)
end

--     
function onGridMouseDown(self, x, y, button)
    if not applyFilter(self.data.group) then 
        return
    end

    selectGroup(self.data.group)
    selectRow(self.data.group, self.data.r)
end

--    
function selectGroup(group)
    if (not group) or (not group.boss) or (not group.boss.boss) or (not applyFilter(group)) then 
        return
    end
    
    if group.hidden then
        MapWindow.unselectAll()
        MapWindow.selectedGroup = group
        setCheckBox(group.type, true)
        local selectedUnit = group.units[1]
        local mapObject = group.mapObjects.units[1]
        MapWindow.selectedUnit = selectedUnit
        MapWindow.respondToSelectedUnit(mapObject, group, selectedUnit)
        return
    end
    
    local mapObject
    
    if group.mapObjects.route then
        mapObject = group.mapObjects.route.points[1]
    else
        mapObject = group.mapObjects.units[1].userObject
        mapObject.userObject = group.mapObjects.units[1].userObject
    end
    
    setCheckBox(group.type, false)
    
    if not mapObject then 
        return
    end
    
    local unit = group.mapObjects.units[1].userObject
    
    MapWindow.unselectAll()
    
    local selectedUnit = group.units[1]
    
    MapWindow.respondToSelectedUnit(mapObject, group, selectedUnit)
    MapWindow.updateSelectedGroup(group)
    panel_route.update()
    
    if window:isVisible() then
        --    
        --           ,
        --        
        MapWindow.setCamera(group.x, group.y)
        
        local mx,my,mw,mh = MapWindow.window:getBounds()
        local _x,_y = MapWindow.window:widgetToWindow((mw - U.right_toolbar_width)/2,(mh - 300)/2)
        local x, y =  MapWindow.getMapPoint(_x,_y)
        
        MapWindow.setCamera(2*group.x - x, 2*group.y - y)        
    end
    
end

function getGroupRowIndex(group)
  local result
  
  for rowIndex = 0, grid:getRowCount() - 1 do 
      local widget = grid:getCell(0, rowIndex)
      
      if widget.data.group == group then 
          result = rowIndex
          
          break
      end
  end
  
  return result
end

--    
function selectRow(group, rowIndex)
    if not applyFilter(group) then 
        grid:selectRow(-1)
        
        return
    end
    
    if not rowIndex then 
        rowIndex = getGroupRowIndex(group)
        
        if rowIndex then
            grid:selectRow(rowIndex)
            selection = rowIndex          
        end
    else
        grid:selectRow(rowIndex)
    end
end

--    
function updateRow(group)
    if not applyFilter(group) then 
        return
    end
    
    local rowIndex = getGroupRowIndex(group)
    
    if rowIndex then
        local widget = grid:getCell(0, rowIndex)
        
        if widget.data.group == group then 
            widget:setText(group.name)
            
            widget = grid:getCell(1, rowIndex)
            widget:setText(group.boss.name)
            
            widget = grid:getCell(2, rowIndex)
            
            local hiddenStr = ''
            
            if group.hidden then 
                hiddenStr = cdata.hidden 
            end
            
            widget:setText(hiddenStr)
            
            widget = grid:getCell(3, rowIndex)
            widget:setText(tostring(#group.units))
        end
    end
end

--      
function setCheckBox(type, b)
    if type == 'plane' or
       type == 'helicopter' then
        panel_aircraft.hiddenCheckbox:setState(b)
    elseif type == 'ship' then
        panel_ship.hiddenCheckbox:setState(b)
    elseif type == 'vehicle' then
        panel_vehicle.hiddenCheckbox:setState(b)
    elseif type == 'static' then
        panel_static.hiddenCheckbox:setState(b)
    end
end

function selectNextGroup()
    local rowCount = grid:getRowCount()
    
    if -1 ~= selection then
        if selection > rowCount - 1 then
            selection = rowCount - 1
        end
        
        if -1 ~= selection and (selection >= 0)then
            local w = grid:getCell(0, selection)
            local data = w.data
            
            selectGroup(data.group)
            selectRow(data.group, selection)
        else
            show(false)
            MapWindow.selectedGroup = nil
            toolbar.untoggle_all_except()        
        end
    end   
end

function saveSelection()
    selection = grid:getSelectedRow()
end

function updateCountriesCombo()
    if MissionModule.mission == nil then
        return
    end
    
    countryFilter:clear()
	
	local countries = CoalitionController.getActiveCountries()
	
    table.insert(countries, 1, cdata.allCountries)
    table.insert(countries, 2, cdata.red)
    table.insert(countries, 3, cdata.blue)	
	
    U.fill_combo(countryFilter, countries)
end 
