base.print(" ")
base.print("===========================================================================================================================================")
base.print(" ")
base.print("Enter ME_MODS script")
base.print(" ")

local print            =       base.print
local pairs            =       base.pairs
local ipairs           =       base.ipairs
local type             =       base.type
local unpack           =       base.unpack
local tostring         =       base.tostring
local debug            =       base.debug
local setmetatable     =       base.setmetatable
local getmetatable     =       base.getmetatable
local loadstring       =       base.loadstring
local loadfile         =       base.loadfile
local setfenv          =       base.setfenv
local string           =       base.string
local table            =       base.table
local io               =       base.io
local math             =       base.math
local lfs              =       base.lfs
local unitsList        =       base.panel_units_list
local MissionModule    =       base.require('me_mission')
local U                =       base.require('me_utilities')
local minizip          =       base.require('minizip')
local MapWindow        =       base.require('me_map_window')
local OptionsData      =       base.require('Options.Data')	-- DCS 1.2.8 and later
--local OptionsData    =       base.require('me_options')	-- DCS 1.2.7 and earlier

----------------------------------------------------------
--
--    159th_Ajax Mission Editor 'Run Script' routines
--
----------------------------------------------------------


-- get options from me_mods_options.lua file

local me_mods_options = {}
local f = loadfile(base.mainPath.."modules/me_mods_options.lua")
local env = {}
if f then
    base.setfenv(f,env)
    f()
    me_mods_options = env.me_mods_options
end
local runScriptOptions = {}
if me_mods_options.runScriptOptions then
    runScriptOptions = me_mods_options.runScriptOptions
else
    runScriptOptions["printUnits"] = false
    runScriptOptions["printCoalitions"] = false
    runScriptOptions["printGroupSummary"] = false
    runScriptOptions["printFlyables"] = false
end


---------------------------------------------------------------------------------------------------------------------------------------------


me_mods = {}

me_mods.printToFile = function(fname, str, mode)
	-- modified from Mist.debug.dump_tbl
	if lfs and io then
		local fdir = lfs.writedir() .. [[Logs\]] .. fname
		local f = io.open(fdir, mode)
		if f then
			f:write(str)
			f:close()
			local msg = 'Printed to ' .. fdir
			print(msg)
		end
	else
		local msg = 'File I/O error.'
		print(msg)
	end
end

me_mods.unitCompare = function(unit1,unit2)
    -- custom table.sort function for units list
    -- sorts by side (desc), group (asc), unit (asc)
    if unit1.coalition == unit2.coalition then
        if unit1.group == unit2.group then
            return unit1.name < unit2.name
        else
            return unit1.group < unit2.group
        end
    else
        return unit1.coalition > unit2.coalition    -- reverse order
    end
end


---------------------------------------------------------------------------------------------------------------------------------------------

-- adapted from ED ME function

me_mods.createGroupByNameTable = function()

    local group_by_name = {}
    for r,group in pairs(MissionModule.group_by_name) do
        if group then 
            table.insert(group_by_name,group) 
        end
    end
    local sortGroupByName = function(group1, group2)
        -- sorts by coalition (desc), groupname (asc)
        if group1.boss.boss.name == group2.boss.boss.name then
            return group1.name < group2.name
        else
            return group1.boss.boss.name > group2.boss.boss.name
        end
--        return group1.name < group2.name
    end
    table.sort(group_by_name, sortGroupByName)
    return group_by_name
	
end


---------------------------------------------------------------------------------------------------------------------------------------------


me_mods.getUnitsTable = function()
    -- build units table with all needed info in one place
    local _tmp = {}
    for unit_id, unit_data in pairs(MissionModule.unit_by_id) do
        if type(unit_data) == 'table' then
            local _n = #_tmp + 1
            _tmp[_n] = {}
            _tmp[_n].index           =    unit_data.index
            _tmp[_n].livery_id       =    unit_data.livery_id or ""
            _tmp[_n].skill           =    unit_data.skill or ""
            _tmp[_n].group           =    unit_data.boss.name
            _tmp[_n].group_type      =    unit_data.boss.type
            _tmp[_n].group_hidden    =    unit_data.boss.hidden or false
            _tmp[_n].country         =    unit_data.boss.boss.name
            _tmp[_n].coalition       =    unit_data.boss.boss.boss.name
            _tmp[_n].type            =    unit_data.type
            _tmp[_n].unitId          =    unit_data.unitId
            _tmp[_n].name            =    unit_data.name
            _tmp[_n].onboard_num     =    unit_data.onboard_num or ""
            _tmp[_n].callsign        =    unit_data.callsign or ""
            _tmp[_n].group_task      =    unit_data.boss.task or ""
            _tmp[_n].x               =    unit_data.x
            _tmp[_n].y               =    unit_data.y
        end
    end
    return _tmp
end


---------------------------------------------------------------------------------------------------------------------------------------------


me_mods.getUnitInfo = function()

    -- To do: re-work to eliminate repetition

    local _tmp = me_mods.getUnitsTable()
    table.sort(_tmp,me_mods.unitCompare)

    print("    AIRCRAFT")
    print(" ")
    local s = string.format("    %-7s%-15s%-27s%-31s%-37s%-12s%-34s%-8s%-5s%-15s%s","SIDE","COUNTRY","GROUP","UNIT","TYPE","SKILL","SKIN","HIDDEN","BORT","CALLSIGN","TASK")
    print(s)
    print(" ")
    for i,v in pairs(_tmp) do
        if i then
            if v.group_type == 'plane' or v.group_type == 'helicopter' then
                s = string.format("    %-7s%-15s%-27s%-31s%-37s%-12s%-34s%-8s%-5s%-15s%s",v.coalition,v.country,v.group,v.name,v.type,v.skill,string.sub(v.livery_id,1,33),tostring(v.group_hidden),v.onboard_num,v.callsign,v.group_task)
                print(s)
            end
        end
    end
    print(" ")
    
    print("    SHIPS")
    print(" ")
    local s = string.format("    %-7s%-15s%-27s%-31s%-37s%-12s%-8s%s","SIDE","COUNTRY","GROUP","UNIT","TYPE","SKILL","HIDDEN","TASK")
    print(s)
    print(" ")
    for i,v in pairs(_tmp) do
        if i then
            if v.group_type == 'ship' then
                s = string.format("    %-7s%-15s%-27s%-31s%-37s%-12s%-8s%s",v.coalition,v.country,v.group,v.name,v.type,v.skill,tostring(v.group_hidden),v.group_task)
                print(s)
            end
        end
    end
    print(" ")
    
    print("    VEHICLES")
    print(" ")
    local s = string.format("    %-7s%-15s%-27s%-31s%-37s%-12s%-8s%s","SIDE","COUNTRY","GROUP","UNIT","TYPE","SKILL","HIDDEN","TASK")
    print(s)
    print(" ")
    for i,v in pairs(_tmp) do
        if i then
            if v.group_type == 'vehicle' then
                s = string.format("    %-7s%-15s%-27s%-31s%-37s%-12s%-8s%s",v.coalition,v.country,v.group,v.name,v.type,v.skill,tostring(v.group_hidden),v.group_task)
                print(s)
            end
        end
    end
    print(" ")
    
    print("    STATICS")
    print(" ")
    local s = string.format("    %-7s%-15s%-27s%-31s%-37s%-8s","SIDE","COUNTRY","GROUP","UNIT","TYPE","HIDDEN")
    print(s)
    print(" ")
    for i,v in pairs(_tmp) do
        if i then
            if v.group_type == 'static' then
                s = string.format("    %-7s%-15s%-27s%-31s%-37s%-8s",v.coalition,v.country,v.group,v.name,v.type,tostring(v.group_hidden))
                print(s)
            end
        end
    end
    print(" ")
end


---------------------------------------------------------------------------------------------------------------------------------------------


me_mods.getGroupInfo = function(printSummary,printList)

    local rowcount = base.panel_units_list.grid:getRowCount()
    local w, group, groupname, country, grouptype, side, skill
    local lvid, utype, bort, callsign
    local nplane = 0
    local nhelicopter = 0
    local nstatic = 0
    local nvehicle = 0
    local nship = 0
    local grp = me_mods.createGroupByNameTable()
    
    if printSummary then
        for i,v in pairs(grp) do
            grouptype = v.type
            if grouptype == 'plane' then nplane = nplane + 1 end
            if grouptype == 'helicopter' then nhelicopter = nhelicopter + 1 end
            if grouptype == 'static' then nstatic = nstatic + 1 end
            if grouptype == 'vehicle' then nvehicle = nvehicle + 1 end
            if grouptype == 'ship' then nship = nship + 1 end
        end
        local ntotal = nplane + nhelicopter + nstatic + nvehicle + nship
        print("   ")
        print(string.format("    %-17s : %+3s","Total groups",ntotal))
        print(string.format("    %-17s : %+3s","Ship groups",nship))
        print(string.format("    %-17s : %+3s","Plane groups",nplane))
        print(string.format("    %-17s : %+3s","Helicopter groups",nhelicopter))
        print(string.format("    %-17s : %+3s","Vehicle groups",nvehicle))
        print(string.format("    %-17s : %+3s","Static groups",nstatic))
    end

    if printList then
        print("  ")
        print(string.format("    %-8s%-36s%-15s%-15s%-11s%-50s%s","SIDE","GROUP","COUNTRY","TYPE","BORT","SKIN","CALLSIGN"))
        print(" ")
        for i,v in pairs(grp) do
            group = v
            groupname = group.name
            grouptype = group.type
            country = group.boss.name
            side = group.boss.boss.name
            lvid = group.units[1].livery_id
            utype = group.units[1].type
            skill = group.units[1].skill
            bort = group.units[1].onboard_num
            callsign = group.units[1].callsign
            if bort == nil then bort = "" end
            if utype == nil then utype = "" end
            if skill == nil then skill = "" end
            if lvid == nil then lvid = "" end
            if (skill == 'Client' or skill == 'Player') and (grouptype == 'plane' or grouptype == 'helicopter') then
                local s
                s = string.format("    %-8s%-36s%-15s%-15s%-11s%-50s%s",side,groupname,country,utype,bort,lvid,callsign)
                print(s)
            end
        end
    end
    if printSummary or printList then print("     ") end
end


---------------------------------------------------------------------------------------------------------------------------------------------


me_mods.getCoalitionInfo = function()
    local coa = MissionModule.mission.coalitions
    print("       ")
    print("    COALITIONS")
    print(" ")
    for i in pairs(coa) do
        for j,v in pairs(coa[i]) do
            local s = string.format("    %-8s%+2s     %s",i,j,v)
            print(s)
        end
    end
    print(" ")
end


---------------------------------------------------------------------------------------------------------------------------------------------


me_mods.reassignCallsigns = function()

    local function tsort(u1,u2)
        if u1.boss.boss.boss.name == u2.boss.boss.boss.name then    -- coalition
            if u1.boss.name == u2.boss.name then    -- group
                return u1.index < u2.index
            else
                return u1.boss.name < u2.boss.name
            end
        else
            return u1.boss.boss.boss.name > u2.boss.boss.boss.name
        end
    end
	
    local _tmp = {}
	
    for i,v in pairs(MissionModule.unit_by_name) do
		if type(v) == 'table' then
			table.insert(_tmp,v)
		end
    end
    table.sort(_tmp,tsort)
    local n = 100
    for unit,unit_data in pairs(_tmp) do
        if type(unit_data) == 'table' then
            if (unit_data.boss.type == 'helicopter' or unit_data.boss.type == 'plane') then
                --if me_mods_options.reassignEasternCallsigns and not U.isWesternCountry(unit_data.boss.boss.name) then -- use options file setting
                if not U.isWesternCountry(unit_data.boss.boss.name) then -- ignore options file setting
                    n = n + 1
                    local oldn = unit_data.callsign
                    unit_data.callsign = n
                end
            end
        end
    end

end



---------------------------------------------------------------------------------------------------------------------------------------------


--    me_mods.setSkin( coalition, aircraftType, skinName )

--    Sets skins for all *client* aircraft of the specified type and coalition. Exceptions are made for the IL-78M and A-50.
--    All skins for a given type are set to the same skin. There is no easy way to programmatically allow for mixed skins.

--    coalition      =   'RED' or 'BLUE'
--    aircraftType   =   'Ka-50', 'UH-1H', 'Mi-8MTV2', etc. Must be the exact name used by DCS (see table below).
--    skinName       =    folder name of the skin files. Note: this is different from the description that appears in the ME drop-down dialog box.

--    Client aircraft names having custom skins

--   "A-10C"
--   "A-50"       * AI exception
--   "F-15C"
--   "IL-78M"     * AI exception
--   "MiG-29S"
--   "P-51D"
--   "Su-25"
--   "Su-25T"
--   "Su-27"
--   "Su-33"
--   "Ka-50"
--   "UH-1H"
--   "Mi-8MTV2"   * Note: "Mi-8MTV2" -- not the usual DCS designation ("Mi-8MT") -- is required here.


me_mods.setSkin = function(coa,ac,skin)
    coa = string.upper(coa)
    ac = string.upper(ac)
    for unit_id, unit_data in pairs(MissionModule.unit_by_id) do
		--print("unitID: "..unit_id.." name: "..(unit_data.name or "nil").." type: "..(unit_data.type or "nil").." skill: "..(unit_data.skill or "nil").." skin: "..(unit_data.skin or "nil"))
        if type(unit_data) == 'table' then
            if string.upper(unit_data.boss.boss.boss.name) == coa then
                if string.upper(unit_data.type or "") == ac then
                    if ac == 'IL-78M' or ac == 'A-50' or string.upper(unit_data.skill or "") == 'CLIENT' then
                        unit_data.livery_id = skin
                    end
                end
            end
        end
    end
end

	
---------------------------------------------------------------------------------------------------------------------------------------------


-- me_mods.setFreq(baseFreq, acType, groupSize, freqIncrement)

-- The default frequency for Ka-50s, 124.000, is now bugged.
-- This function was created to easily change the frequency of all Ka-50s in the mission.
-- However, its use is not restricted to Ka-50s -- the frequency for any a/c type can be set.

-- baseFreq      = starting frequency
-- acType        = aircraft type (see above for listing of a/c types)
-- groupSize     = number of units to get the same frequency
-- freqIncrement = the amount to add to the base frequency after every nth unit where n = 'groupSize'

-- Each coalition is treated separately.
-- Operates on client units only.


me_mods.setFreq = function(baseFreq, acType, groupSize, freqIncrement)

    local function tsort(g1,g2)
        if g1.boss.boss.name == g2.boss.boss.name then    -- coalition name
			return g1.name < g2.name -- group name
        else
			return g1.boss.boss.name > g2.boss.boss.name -- red gets put at top of list
        end
    end

	local clients = {}
	clients['red'] = {}
	clients['blue'] = {}
    
    for unit_id, unit_data in pairs(MissionModule.unit_by_id) do
        if type(unit_data) == 'table' then
            if string.upper(unit_data.type or "") == string.upper(acType) and string.upper(unit_data.skill or "") == 'CLIENT' then
				table.insert(clients[string.lower(unit_data.boss.boss.boss.name)],unit_data.boss) -- group table
			end
		end
	end

	table.sort(clients['red'],tsort)
	table.sort(clients['blue'],tsort)

	for coa,coa_data in pairs(clients) do
		local cntr = 0
		for group_id, group_data in pairs(coa_data) do
			local mult = math.floor(cntr/groupSize)
			group_data.frequency = baseFreq + mult * freqIncrement
			cntr = cntr + 1
			print("setFreq "..coa.." "..group_data.units[1].type)
		end
	end

end

   
---------------------------------------------------------------------------------------------------------------------------------------------


-- me_mods.setOptions(mOptions)
--
-- Sets/clears mission options.
--
-- mOptions  =  Lua table containing the forced options defined below.
--
-- Comment out each line you do *not* want to enforce.
-- Uncomment and set the value for each one you *do* want to enforce.


me_mods.setOptions = function(mOptions)
	
	mOptions = mOptions or {}
    OptionsData.setForcedOptions(mOptions)

end

local mOptions = {}
  
  mOptions.padlock                 = false
  mOptions.easyRadar               = false
  mOptions.scenes                  = "high"                   --  "low", "medium", "high"
  mOptions.birds                   = 0                        --   numeric: 0 to 1000
  mOptions.optionsView             = "optview_onlyallies"     --  "optview_onlymap", "optview_myaircraft", "optview_allies", "optview_onlyallies", "optview_all"
  mOptions.permitCrash             = false                        
  mOptions.immortal                = false                        
  mOptions.easyCommunication       = false                        
  mOptions.easyFlight              = false                        
  mOptions.radio                   = false                        
--mOptions.tips                    = false                    --   this option will be set to un-forced
  mOptions.accidental_failures     = false                        
  mOptions.fuel                    = false                        
  mOptions.externalViews           = true                         
  mOptions.geffect                 = "realistic"              --  "none", "reduced", "realistic"
  mOptions.civTraffic              = ""                       --  "", "low", "medium", "high"
  mOptions.weapons                 = false
  mOptions.miniHUD                 = false
  mOptions.labels                  = false
--mOptions.cockpitVisualRM         = true                     --   also set to un-forced
  
 
---------------------------------------------------------------------------------------------------------------------------------------------


-- Print Red and Blue AD units

me_mods.getAdUnits = function(coa) 

	local units = me_mods.getUnitsTable()

	-- get AD units from ME
	local gndUnits = base.me_db.car_by_name
	local adUnits = {}
	for k,v in pairs(gndUnits) do
		if v.attribute and type(v.attribute) == 'table' then
			for i = 1, #v.attribute do
				if v.attribute[i] == 'Air Defence' then
					adUnits[k] = true
					break
				end
			end
		end
	end

--	mist.debug.dump_tbl(adUnits,'AD_Units.lua')
	
--[[
	for k,v in pairs(adUnits) do
		print(tostring(k))
	end
--]]
	
    local function tsort(u1,u2)
		if u1.lat == u2.lat then
			return u1.long < u2.long
		else
			return u1.lat > u2.lat
		end
    end
	
	t = {}
	local i
	local c = 0
	for i=1,#units do
		if coa == units[i].coalition and adUnits[units[i].type] then
			c = c + 1
			local lat, long = MapWindow.convertMetersToLatLon(units[i].x, units[i].y)
			t[c] = {}
			t[c].name = units[i].name
			t[c].type = units[i].type
			t[c].lat  = lat
			t[c].long = long
		end
	end
	table.sort(t,tsort)
	for i=1,#t do
		local dlat  = math.floor(t[i].lat)
		local dlong = math.floor(t[i].long)
		local mlat  = (t[i].lat - dlat)*60
		local mlong = (t[i].long - dlong)*60
		print(string.format("    %-6s%6d     %-30s%10s %2d %05.2f  %s %03d %05.2f ",coa,i,t[i].type,"Lat = ",dlat,mlat,"Long = ",dlong,mlong))
	end
	
end


---------------------------------------------------------------------------------------------------------------------------------------------

-- Prints current list of available units in the ME

me_mods.printMEunits = function(categorized, sortBy, outFile, printInfo)

	-- sortBy      = "NAME" or "DISPLAYNAME"
	-- categorized = true or false
	-- outFile     = filename to write info
	-- printInfo   = true or false
	
	local subTableName = {
		['Heliports']      = 'Heliport'      ,
		['Ships']          = 'Ship'          ,
		['Cargos']         = 'Cargo'         ,
		['Helicopters']    = 'Helicopter'    ,
		['Fortifications'] = 'Fortification' ,
		['Cars']           = 'Car'           ,
		['Warehouses']     = 'Warehouse'     ,
		['Planes']         = 'Plane'         ,
	}

	local tsort = function(t1,t2)
		if categorized then
			if t1.category == t2.category then
				if sortBy == "NAME" then
					return string.upper(t1.name) < string.upper(t2.name)
				else
					return string.upper(t1.displayName) < string.upper(t2.displayName)
				end
			else
				return t1.category < t2.category
			end
		else
			if sortBy == "NAME" then
				return string.upper(t1.name) < string.upper(t2.name)
			else
				return string.upper(t1.displayName) < string.upper(t2.displayName)
			end
		end
	end
	
	local tmp = {}
	local counter = 0
    for i,v in pairs(base.me_db.db.Units) do
		if i ~= 'GT_t' and i ~= 'Skills' and i ~= 'GroundObjects' then
			local t = v[ subTableName[i] ]
			for j = 1, #t do
				counter = counter + 1
				local displayName = t[j].Name
				local name = tostring(base.me_db.nameByDisplayName[displayName])
				tmp[counter] = {["category"] = i,["displayName"] = displayName,["name"] = name}
			end
		end
	end

	table.sort(tmp,tsort)

	local s = ""
	local sFile = ""
	local fmtstr = ""
	if categorized then
		fmtstr = "    %-18s%-36s%-36s"
		if sortBy == "NAME" then
			s = string.format(fmtstr,"CATEGORY","NAME (SSE)","DISPLAY NAME (ME)")
		else
			s = string.format(fmtstr,"CATEGORY","DISPLAY NAME (ME)","NAME (SSE)")
		end
		if printInfo then print(s) end
		sFile = s .. "\r\n"
		for i = 1, #tmp do
			if sortBy == "NAME" then
				s = string.format(fmtstr,tmp[i].category,tmp[i].name,tmp[i].displayName)
			else
				s = string.format(fmtstr,tmp[i].category,tmp[i].displayName,tmp[i].name)
			end
			if printInfo then print(s) end
			sFile = sFile .. s .. "\r\n"
		end
	else
		fmtstr = "    %-36s%-36s%-18s"
		if sortBy == "NAME" then
			s = string.format(fmtstr,"NAME (SSE)","DISPLAY NAME (ME)","CATEGORY")
		else
			s = string.format(fmtstr,"DISPLAY NAME (ME)","NAME (SSE)","CATEGORY")
		end
		if printInfo then print(s) end
		sFile = s .. "\r\n"
		for i = 1, #tmp do
			if sortBy == "NAME" then
				s = string.format(fmtstr,tmp[i].name,tmp[i].displayName,tmp[i].category)
			else
				s = string.format(fmtstr,tmp[i].displayName,tmp[i].name,tmp[i].category)
			end
			if printInfo then print(s) end
			sFile = sFile .. s .. "\r\n"
		end
	end
	me_mods.printToFile(outFile,sFile,'w')

end


---------------------------------------------------------------------------------------------------------------------------------------------


me_mods.findMapClassKey = function()
	local db = base.me_db
	local units = me_mods.getUnitsTable()
	for i=1,#units do
		if units[i].type ~= nil then
			if db.unit_by_name[units[i].type] then
				if db.unit_by_name[units[i].type].mapclasskey == nil then
					print(units[i].name.." ["..units[i].type.."] has no mapclasskey")
				end
			else
				print(units[i].name.." type not found")
			end
		else
			print(units[i].name.." has no type")
		end
	end
end


---------------------------------------------------------------------------------------------------------------------------------------------


me_mods.assignAIskill = function(coa, unitCategory, skillLevel)

-- unitCategory
--		'plane'
--		'helicopter'
--		'vehicle'
--		'ship'

-- coa
--		'BLUE'
--		'RED'

-- skillLevel
--		'Random'
--		'Average'
--		'Good'
--		'High'
--		'Exellent'


    for unit_id, unit_data in pairs(MissionModule.unit_by_id) do
        if type(unit_data) == 'table' then
			if coa == unit_data.boss.boss.boss.name and unitCategory == unit_data.boss.type and unit_data.skill ~= 'Client' and unit_data.skill ~= 'Player' then
				unit_data.skill = skillLevel
			end
        end
    end
end


---------------------------------------------------------------------------------------------------------------------------------------------


me_mods.printUnitStartTime = function()

    local tsort = function(u1, u2)
        return u1.name < u2.name
    end
	local c = 0
	tmp = {}
    for unit_id, unit_data in pairs(MissionModule.unit_by_id) do
        if type(unit_data) == 'table' then
			if unit_data.boss.type ~= 'static' then
				c = c + 1
				tmp[c] = {}
				tmp[c].name = unit_data.name
				tmp[c].boss = unit_data.boss.name
				tmp[c].start_time = unit_data.boss.start_time or "---"
				local late = unit_data.boss.lateActivation or "---"
				local player = unit_data.playerCanDrive or "---"
				if late == true then late = "Late Activated" end
				tmp[c].lateActivation = tostring(late)
				if player == true then player = "Player can drive" end
				tmp[c].playerCanDrive = player
			end
        end
    end
	table.sort(tmp,tsort)
	for i=1,#tmp do
		s = string.format(" %-46s%-46s%-15s%-20s%-15s",tmp[i].name,tmp[i].boss,tostring(tmp[i].start_time),tostring(tmp[i].lateActivation),tostring(tmp[i].playerCanDrive))
		print(s)
	end
end


---------------------------------------------------------------------------------------------------------------------------------------------


me_mods.printCallnames = function()
	local me_db = base.me_db
	local db = me_db.db
	if not db then print("Cannot load db") return end
	local fmtstr = "    %-15s%-25s%-30s"
	print("  ")
	print("    CALLNAMES")
	print(" ")
	print(string.format(fmtstr,"COUNTRY","TYPE","CALLSIGN"))
	print(" ")
	for countryID, callsignType in pairs(db.Callnames) do
		for i,v in pairs(callsignType) do
			for j,w in pairs(v) do
				s = string.format(fmtstr,me_db.country_by_id[countryID].Name,i,w.Name)
				print(s)
			end
		end
	end
end


---------------------------------------------------------------------------------------------------------------------------------------------


-- Run various routines enabled in options file
-- Easier to use the standalone versions at end of file

---[[
if me_mods_options.reassignCallsigns then me_mods.reassignCallsigns() end
if runScriptOptions.printCoalitions then me_mods.getCoalitionInfo() end
if runScriptOptions.printGroupSummary or runScriptOptions.printFlyables then me_mods.getGroupInfo(runScriptOptions.printGroupSummary,runScriptOptions.printFlyables) end
if runScriptOptions.printUnits then me_mods.getUnitInfo() end
--]]

---------------------------------------------------------------------------------------------------------------------------------------------


--Uncomment the following block to assign AI skill level for all AI units of the given type. Change as needed.
--[[
	me_mods.assignAIskill( 'BLUE' , 'helicopter' , 'Random' )
	me_mods.assignAIskill( 'BLUE' , 'plane'      , 'Random' )
	me_mods.assignAIskill( 'BLUE' , 'vehicle'    , 'Random' )
	me_mods.assignAIskill( 'BLUE' , 'ship'       , 'Random' )
	me_mods.assignAIskill( 'RED'  , 'helicopter' , 'Random' )
	me_mods.assignAIskill( 'RED'  , 'plane'      , 'Random' )
	me_mods.assignAIskill( 'RED'  , 'vehicle'    , 'Random' )
	me_mods.assignAIskill( 'RED'  , 'ship'       , 'Random' )
--]]

--Prints a listing of air-defense units in the mission with lat/long coordinates
--  me_mods.getAdUnits("RED")
--  me_mods.getAdUnits("BLUE")

--Use the following as a guide to set the skins of various sides/aircraft type.
--[[
    me_mods.setSkin(  "RED"   ,  "Ka-50"     ,   "Russia Standard Army (Worn)"         )
    me_mods.setSkin(  "RED"   ,  "UH-1H"     ,   "159th GAR splinter"                  )
--  me_mods.setSkin(  "RED"   ,  "UH-1H"     ,   "159th GAR"                           )
    me_mods.setSkin(  "RED"   ,  "Su-25"     ,   "159th SU-25 Splinter Camo"           )
--  me_mods.setSkin(  "RED"   ,  "Su-25"     ,   "159th SU-25"                         )
    me_mods.setSkin(  "RED"   ,  "Su-25T"    ,   "159th GAR"                           )
--  me_mods.setSkin(  "RED"   ,  "Su-25T"    ,   "159th GAR Salmon"                    )
--  me_mods.setSkin(  "RED"   ,  "Su-25T"    ,   "159th GAR Splinter Camo W"           )
    me_mods.setSkin(  "RED"   ,  "A-10C"     ,   "159th GAR A-10C"                     )
    me_mods.setSkin(  "RED"   ,  "Mi-8MTV2"  ,   "159th_GAR Splinter"                  )
--  me_mods.setSkin(  "RED"   ,  "Mi-8MTV2"  ,   "159th_GAR curvy"                     )
--  me_mods.setSkin(  "RED"   ,  "Mi-8MTV2"  ,   "159th_GAR curvy matt"                )
    me_mods.setSkin(  "RED"   ,  "IL-78M"    ,   "159th GAR"                           )
    me_mods.setSkin(  "RED"   ,  "MiG-29S"   ,   "115th guards regiment, termez ab"    )
    me_mods.setSkin(  "RED"   ,  "Su-27"     ,   "159th GAR splinter"                  )
    me_mods.setSkin(  "RED"   ,  "Su-33"     ,   "159th GAR"                           )
    me_mods.setSkin(  "RED"   ,  "P-51D"     ,   "Russia 159th GAR"                    )
    me_mods.setSkin(  "RED"   ,  "A-50"      ,   "159th GAR"                           )
--  me_mods.setSkin(  "BLUE"  ,  "F-15C"     ,   "159th Agressor"                      )
--  me_mods.setSkin(  "BLUE"  ,  "Ka-50"     ,   "Russia Standard Army (Worn)"         )
--  me_mods.setSkin(  "BLUE"  ,  "UH-1H"     ,   "159th GAR splinter"                  )
--  me_mods.setSkin(  "BLUE"  ,  "UH-1H"     ,   "159th GAR"                           )
--  me_mods.setSkin(  "BLUE"  ,  "Su-25"     ,   "159th SU-25 Splinter Camo"           )
--  me_mods.setSkin(  "BLUE"  ,  "Su-25"     ,   "159th SU-25"                         )
--  me_mods.setSkin(  "BLUE"  ,  "Su-25T"    ,   "159th GAR"                           )
--  me_mods.setSkin(  "BLUE"  ,  "Su-25T"    ,   "159th GAR Salmon"                    )
--  me_mods.setSkin(  "BLUE"  ,  "Su-25T"    ,   "159th GAR Splinter Camo W"           )
--  me_mods.setSkin(  "BLUE"  ,  "A-10C"     ,   "159th GAR A-10C"                     )
--  me_mods.setSkin(  "BLUE"  ,  "Mi-8MTV2"  ,   "159th_GAR Splinter"                  )
--  me_mods.setSkin(  "BLUE"  ,  "Mi-8MTV2"  ,   "159th_GAR curvy"                     )
--  me_mods.setSkin(  "BLUE"  ,  "Mi-8MTV2"  ,   "159th_GAR curvy matt"                )
--  me_mods.setSkin(  "BLUE"  ,  "IL-78M"    ,   "159th GAR"                           )
--  me_mods.setSkin(  "BLUE"  ,  "MiG-29S"   ,   "115th guards regiment, termez ab"    )
--  me_mods.setSkin(  "BLUE"  ,  "Su-27"     ,   "159th GAR splinter"                  )
--  me_mods.setSkin(  "BLUE"  ,  "Su-33"     ,   "standard-2 navy"                     )
--  me_mods.setSkin(  "BLUE"  ,  "P-51D"     ,   "Russia 159th GAR"                    )
--  me_mods.setSkin(  "BLUE"  ,  "A-50"      ,   "159th GAR"                           )
    me_mods.setSkin(  "BLUE"  ,  "F-15C"     ,   "159th Agressor"                      )
--]]

    me_mods.setFreq( 127  , 'Ka-50' , 4 , 0.5 )
    me_mods.setFreq( 251  , 'UH-1H' , 4 , 1   )
    me_mods.setOptions(mOptions)

	
--The following optionally prints and outputs lua files with all of the current unit types in both mission editor name and scripting engine names. Useful for scripters.
--[[me_mods.printMEunits( categorized , sortBy , outputFile , printInfo )--]]
	me_mods.printMEunits( true  , "DISPLAYNAME" , "ME_UnitsMEname_cat.lua"    , false )
	me_mods.printMEunits( true  , "NAME"        , "ME_UnitsSSEname_cat.lua"   , false )
	me_mods.printMEunits( false , "DISPLAYNAME" , "ME_UnitsMEname_uncat.lua"  , false )
	me_mods.printMEunits( false , "NAME"        , "ME_UnitsSSEname_uncat.lua" , false )
	
--  me_mods.printUnitStartTime()		-- useful in updating older missions which used a delayed start time instead of 'late activated'


--  Built-in functions to be used as needed without relying on options file settings
--  me_mods.reassignCallsigns()
--  me_mods.getCoalitionInfo()
--	me_mods.getUnitInfo()
--  me_mods.getGroupInfo(true,true)	-- arg 1 = print summary; arg 2 = print flyables

--  me_mods.printCallnames()

---------------------------------------------------------------------------------------------------------------------------------------------

print(" ")
print("Exit ME_MODS script")
